@@ -102,7 +102,9 @@ class RaidMember(BasePlayer):
 
     @property
     def attacks(self):
-        """List[:class:`RaidAttack`]: The member's attacks in this raid log entry"""
+        """List[:class:`RaidAttack`]: The member's attacks in this raid log entry.
+        Can be empty due to missing parts in the API response.
+        """
         list_attacks = self._attacks  # type: List[RaidAttack]
         if list_attacks:
             return list_attacks
@@ -131,6 +133,8 @@ class RaidAttack:
         :class:`RaidClan` - The raid clan this attack belongs to
     district:
         :class:`RaidDistrict` - The raid district this attack belongs to
+    stars:
+        :class:`int` - The raid attacks stars
     """
 
     __slots__ = ("raid_log_entry",
@@ -140,7 +144,10 @@ class RaidAttack:
                  "attacker_tag",
                  "attacker_name",
                  "destruction",
-                 "_client")
+                 "stars",
+                 "_client",
+                 "_raw_data",
+                 )
 
     def __repr__(self):
         attrs = [
@@ -149,6 +156,7 @@ class RaidAttack:
             ("district", repr(self.district)),
             ("attacker_tag", self.attacker_tag),
             ("destruction", self.destruction),
+            ("stars", self.stars),
         ]
         return "<%s %s>" % (self.__class__.__name__, " ".join("%s=%r" % t for t in attrs),)
 
@@ -156,6 +164,7 @@ class RaidAttack:
         if isinstance(other, RaidAttack):
             if (self.attacker_tag == other.attacker_tag
                 and self.destruction == other.destruction
+                and self.stars == other.stars
                 and self.raid_clan == other.raid_clan
                 and self.district == other.district
             ):
@@ -167,6 +176,7 @@ class RaidAttack:
         self.raid_clan = raid_clan
         self.district = district
         self._client = client
+        self._raw_data = data if client and client.raw_attribute else None
         self._from_data(data)
 
     def _from_data(self, data: dict) -> None:
@@ -227,7 +237,8 @@ class RaidDistrict:
                  "attacks",
                  "raid_log_entry",
                  "raid_clan",
-                 "_client")
+                 "_client",
+                 "_raw_data")
 
     def __str__(self):
         return self.name
@@ -250,6 +261,7 @@ class RaidDistrict:
         self.looted: int = data.get("totalLooted")
         self.raid_log_entry = raid_log_entry  # type: RaidLogEntry
         self.raid_clan = raid_clan  # type: RaidClan
+        self._raw_data = data if client and client.raw_attribute else None
         self._client = client
         if data.get("attacks", None):
             self.attacks: List[RaidAttack] = [RaidAttack(data=adata, client=client,
@@ -279,6 +291,8 @@ class RaidClan:
             The number of districts in the raid.
         destroyed_district_count: :class:`int`
             The number of destroyed districts in the raid.
+        index:
+            :class:`int` - The index/order of the raid clan in the raid weekend
         raid_log_entry:
             :class:`RaidLogEntry` - The raid log entry this attack belongs to
         """
@@ -288,6 +302,7 @@ class RaidClan:
         "name",
         "badge",
         "level",
+        "index",
         "attack_count",
         "district_count",
         "destroyed_district_count",
@@ -297,12 +312,13 @@ class RaidClan:
         "_client",
         "_response_retry",
         "_cs_raid_districts",
-        "_iter_raid_districts"
+        "_iter_raid_districts",
+        "_raw_data"
     )
 
-    def __init__(self, *, data, client, raid_log_entry, **_):
+    def __init__(self, *, data, client, raid_log_entry, index=0, **_):
         self._client = client
-
+        self._raw_data = data if client and client.raw_attribute else None
         self._response_retry = data.get("_response_retry")
         self.tag = data.get("attacker", data.get("defender")).get("tag")
         self.name = data.get("attacker", data.get("defender")).get("name")
@@ -310,6 +326,7 @@ class RaidClan:
                               client=self._client)
         self.level = data.get("attacker", data.get("defender")).get("level")
         self.raid_log_entry = raid_log_entry
+        self.index = index
         self._attacks = []
         self._from_data(data)
 
@@ -320,7 +337,7 @@ class RaidClan:
                 and self.district_count == other.district_count
                 and self.destroyed_district_count == other.destroyed_district_count
                 and self.raid_log_entry.start_time == other.raid_log_entry.start_time
-                and self.attacks == other.attacks)
+                and self.index == other.index)
 
     def __repr__(self):
         attrs = [
@@ -368,7 +385,7 @@ class RaidLogEntry:
     Attributes
     ----------
     state:
-        :class:`str`: The state of the raid log entry
+        :class:`str`: The state of the raid log entry. Currently, the states ``ongoing`` and ``ended`` are known.
     start_time:
         :class:`Timestamp`: The :class:`Timestamp` that the raid started at.
     end_time:
@@ -396,7 +413,7 @@ class RaidLogEntry:
                  "destroyed_district_count",
                  "offensive_reward",
                  "defensive_reward",
-
+                 "_raw_data",
                  "_cs_attack_log",
                  "_cs_defense_log",
                  "_cs_members",
@@ -407,11 +424,13 @@ class RaidLogEntry:
                  "_attack_log",
                  "_defense_log",
                  "_client",
-                 "_response_retry")
+                 "_response_retry"
+                 )
 
-    def __init__(self, *, data, client, **_):
+    def __init__(self, *, data, client, **kwargs):
         self._client = client
-        self._response_retry = data.get("_response_retry")
+        self._response_retry = kwargs['response_retry'] if "response_retry" in kwargs else 0
+        self._raw_data = data if client and client.raw_attribute else None
         self._from_data(data)
         self._members = {}
         self._attack_log = []
@@ -451,10 +470,11 @@ class RaidLogEntry:
         self.offensive_reward: int = data_get("offensiveReward")
         self.defensive_reward: int = data_get("defensiveReward")
 
-        self._iter_attack_log = (RaidClan(data=adata, raid_log_entry=self, client=self._client)
-                                 for adata in data_get("attackLog", []))
-        self._iter_defense_log = (RaidClan(data=adata, raid_log_entry=self, client=self._client)
-                                  for adata in data_get("defenseLog", []))
+        self._iter_attack_log = (RaidClan(data=adata, raid_log_entry=self, client=self._client, index=c)
+                                 for c, adata in enumerate(data_get("attackLog", [])))
+
+        self._iter_defense_log = (RaidClan(data=adata, raid_log_entry=self, client=self._client, index=c)
+                                  for c, adata in enumerate(data_get("defenseLog", [])))
 
         self._iter_members = (RaidMember(data=adata, raid_log_entry=self, client=self._client)
                               for adata in data_get("members", []))
@@ -470,15 +490,15 @@ class RaidLogEntry:
     def attack_log(self) -> typing.List[RaidClan]:
         """List[:class:`RaidClan`]: A list of raid clans that are attacked in the raid season.
         """
-        dict_attack_log = self._attack_log = {m.tag: m for m in self._iter_attack_log}
-        return list(dict_attack_log.values())
+        list_attack_log = self._attack_log = [m for m in self._iter_attack_log]
+        return list_attack_log
 
     @cached_property("_cs_defense_log")
     def defense_log(self) -> typing.List[RaidClan]:
         """List[:class:`RaidClan`]: A list of raid clans which represents all the defensive raids of a season.
         """
-        dict_defense_log = self._defense_log = {m.tag: m for m in self._iter_defense_log}
-        return list(dict_defense_log.values())
+        list_defense_log = self._defense_log = [m for m in self._iter_defense_log]
+        return list_defense_log
 
     def get_member(self, tag: str) -> typing.Optional[RaidMember]:
         """Get a member of the clan for the given tag, or ``None`` if not found.
